unit gzipWrap;

interface

uses
  SDUGeneral,
  zlibex, zlibexgz;

function gzip_Compression(
              source: string;
              destination: string;
              compressNotDecompress: boolean;
              compressionLevel: TZCompressionLevel = zcNone;
              startOffset: int64 = 0;  // Only valid when compressing
              compressedFilename: string = 'compressed.dat';  // Only valid when compressing
              length: int64 = -1;
              blocksize: int64 = 4096;
              callback: TCopyProgressCallback = nil
             ): boolean;

function GZLibCompressionLevelTitle(compressionLevel: TZCompressionLevel): string;

implementation

uses
  Windows, Classes, sysutils;


resourcestring
  ZCCOMPRESSLVL_NONE    = 'None';
  ZCCOMPRESSLVL_FASTEST = 'Fastest';
  ZCCOMPRESSLVL_DEFAULT = 'Default';
  ZCCOMPRESSLVL_MAX     = 'Max';
  ZCCOMPRESSLVL_LEVEL1  = 'Level1';
  ZCCOMPRESSLVL_LEVEL2  = 'Level2';
  ZCCOMPRESSLVL_LEVEL3  = 'Level3';
  ZCCOMPRESSLVL_LEVEL4  = 'Level4';
  ZCCOMPRESSLVL_LEVEL5  = 'Level5';
  ZCCOMPRESSLVL_LEVEL6  = 'Level6';
  ZCCOMPRESSLVL_LEVEL7  = 'Level7';
  ZCCOMPRESSLVL_LEVEL8  = 'Level8';
  ZCCOMPRESSLVL_LEVEL9  = 'Level9';

const
  GZLibCompressionLevelTitlePtr: array [TZCompressionLevel] of Pointer = (
                                  @ZCCOMPRESSLVL_NONE,
                                  @ZCCOMPRESSLVL_FASTEST,
                                  @ZCCOMPRESSLVL_DEFAULT,
                                  @ZCCOMPRESSLVL_MAX,
                                  @ZCCOMPRESSLVL_LEVEL1,
                                  @ZCCOMPRESSLVL_LEVEL2,
                                  @ZCCOMPRESSLVL_LEVEL3,
                                  @ZCCOMPRESSLVL_LEVEL4,
                                  @ZCCOMPRESSLVL_LEVEL5,
                                  @ZCCOMPRESSLVL_LEVEL6,
                                  @ZCCOMPRESSLVL_LEVEL7,
                                  @ZCCOMPRESSLVL_LEVEL8,
                                  @ZCCOMPRESSLVL_LEVEL9
                                 );

function GZLibCompressionLevelTitle(compressionLevel: TZCompressionLevel): string;
begin
  Result := LoadResString(GZLibCompressionLevelTitlePtr[compressionLevel]);
end;


// ----------------------------------------------------------------------------
// compressNotDecompress - Set to TRUE to compress, FALSE to decompress
// compressionLevel - Only used if compressNotDecompress is TRUE
// length - Set to -1 to copy until failure
function gzip_Compression(
              source: string;
              destination: string;
              compressNotDecompress: boolean;
              compressionLevel: TZCompressionLevel = zcNone;
              startOffset: int64 = 0;  // Only valid when compressing
              compressedFilename: string = 'compressed.dat';  // Only valid when compressing
              length: int64 = -1;
              blocksize: int64 = 4096;
              callback: TCopyProgressCallback = nil
             ): boolean;
var
  allOK: boolean;
  numberOfBytesRead, numberOfBytesWritten: DWORD;
  finished: boolean;
  copyCancelFlag: boolean;
  totalBytesCopied: int64;
  stmFileInput: TFileStream;
  stmFileOutput: TFileStream;
  stmInput: TStream;
  stmOutput: TStream;
  stmCompress: TGZCompressionStream;
  stmDecompress_GZ: TGZDecompressionStream;
  stmDecompress_Z: TZDecompressionStream;
  buffer: PChar;
  useGZNotZDecompression: boolean;
begin
//  CompFileStream := TFileStream.Create(source, fmOpenRead);// OR fmShareDenyWrite);
//  FileStream := TFileStream.Create(destination, fmCreate);

  allOK := TRUE;
  copyCancelFlag := FALSE;

  useGZNotZDecompression := TRUE;
  stmCompress := nil;
  stmDecompress_GZ := nil;
  stmDecompress_Z := nil;

  try
    stmFileInput := TFileStream.Create(source, fmOpenRead);
  except
    stmFileInput := nil;
  end;
  if (stmFileInput = nil) then
    begin
    raise EExceptionBadSrc.Create('Can''t open source');
    end;

  try
    try
      try
        // Try to create new...
        stmFileOutput := TFileStream.Create(destination, fmCreate);
      except
        // Fallback to trying to open existing for write...
        stmFileOutput := TFileStream.Create(destination, fmOpenWrite);
      end;
    except
      stmFileOutput := nil;
    end;
    if (stmFileOutput = nil) then
      begin
      raise EExceptionBadDest.Create('Can''t open destination');
      end;

    try
      if compressNotDecompress then
        begin
        stmCompress:= TGZCompressionStream.Create(stmFileOutput, compressedFilename, '', Now(), compressionLevel);

        stmInput:= stmFileInput;
        stmOutput:= stmCompress;

        if (length < 0) then
          begin
          length := stmFileInput.Size;
          end;

        stmInput.Position := startOffset;
        end
      else
        begin
        // v2.00 of USB Flash Tools created compressed files WITHOUT the gzip
        // header, causing problems when decompressing
        // To cater for compressed images created without this header, we try
        // to create a decompressor, assuming it's a standard .gz file, then
        // fallback to trying TZDecompressionStream (i.e. without a header)
        try
          stmDecompress_GZ:= TGZDecompressionStream.Create(stmFileInput);
          stmInput:= stmDecompress_GZ;
          useGZNotZDecompression := TRUE;
        except
          stmDecompress_GZ := nil;
          
          stmFileInput.Position := 0;
          stmDecompress_Z:= TZDecompressionStream.Create(stmFileInput);
          stmInput:= stmDecompress_Z;
          useGZNotZDecompression := FALSE;
        end;

        stmOutput:= stmFileOutput;

//        length := stmDecompress.Size;
//        length := 0;
//        stmDecompress.Position := 0;
        blocksize := 4096;  // Decompressing requires a blocksize of 4096?!
        end;

      totalBytesCopied := 0;
      if assigned(callback) then
        begin
        callback(totalBytesCopied, copyCancelFlag);
        end;


      buffer := AllocMem(blocksize);
      try
        finished := FALSE;
        while not(finished) do
          begin
          if compressNotDecompress then
            begin
            numberOfBytesRead := stmInput.Read(buffer^, blocksize);
            end
          else
            begin
            if useGZNotZDecompression then
              begin
              numberOfBytesRead := stmDecompress_GZ.Read(buffer^, blocksize);
              end
            else
              begin
              numberOfBytesRead := stmDecompress_Z.Read(buffer^, blocksize);
              end;
            end;

          // If we read was successful, and we read some bytes, we haven't
          // finished yet... 
          finished := (numberOfBytesRead <= 0);

          if (numberOfBytesRead>0) then
            begin
            // If we've got a limit to the number of bytes we should copy...
            if (length >= 0) then
              begin
              if ((totalBytesCopied+numberOfBytesRead) > length) then
                begin
                numberOfBytesRead := length - totalBytesCopied;
                end;
              end;

            numberOfBytesWritten := stmOutput.Write(buffer^, numberOfBytesRead);

            if (numberOfBytesRead <> numberOfBytesWritten) then
              begin
              raise EExceptionWriteError.Create('Unable to write data to output');
              end;

            totalBytesCopied := totalBytesCopied + numberOfBytesWritten;
            if assigned(callback) then
              begin
              callback(totalBytesCopied, copyCancelFlag);
              end;

            if (
                (length >= 0) and
                (totalBytesCopied >= length)
               ) then
              begin
              finished := TRUE;
              end;

            // Check for user cancel
            if copyCancelFlag then
              begin
              raise EExceptionUserCancel.Create('User cancelled operation.');
              end;

            end;
          end;
          
      finally
        FreeMem(buffer);
      end;

    finally
      if (
          compressNotDecompress and
          (stmCompress <> nil)
         ) then
        begin
        stmCompress.Free();
        end;
      stmFileOutput.Free();
    end;

  finally
    if (
        not(compressNotDecompress) and
        (stmDecompress_GZ <> nil)
       ) then
      begin
      stmDecompress_GZ.Free();
      end;

    if (
        not(compressNotDecompress) and
        (stmDecompress_Z <> nil)
       ) then
      begin
      stmDecompress_Z.Free();
      end;

    stmFileInput.Free();
  end;


  Result := allOK;
end;

END.

